@@ -3,11 +3,15 @@
 
 #include "glf.h"
 
+#define BAM_ERRMOD_MAQ2 0
+#define BAM_ERRMOD_MAQ  1
+#define BAM_ERRMOD_SOAP 2
+
 struct __bmc_aux_t;
 
 typedef struct {
 	float het_rate, theta;
-	int n_hap, cap_mapQ, is_soap;
+	int n_hap, cap_mapQ, errmod, min_baseQ;
 
 	float eta, q_r;
 	double *fk, *coef;
@@ -16,8 +20,9 @@ typedef struct {
 } bam_maqcns_t;
 
 typedef struct {
-	int q_indel;
-	float r_indel;
+	int q_indel; // indel sequencing error, phred scaled
+	float r_indel; // indel prior
+	float r_snp; // snp prior
 	// hidden parameters, unchangeable from command line
 	int mm_penalty, indel_err, ambi_thres;
 } bam_maqindel_opt_t;
